@@ -16,7 +16,8 @@ from ..errors import EWSWarning, TransportError, SOAPError, ErrorTimeoutExpired,
     ErrorCannotDeleteTaskOccurrence, ErrorMimeContentConversionFailed, ErrorRecurrenceHasNoOccurrence, \
     ErrorNoPublicFolderReplicaAvailable, MalformedResponseError, ErrorExceededConnectionCount, \
     SessionPoolMinSizeReached, ErrorIncorrectSchemaVersion, ErrorInvalidRequest, ErrorCorruptData, \
-    ErrorCannotEmptyFolder, ErrorDeleteDistinguishedFolder, ErrorInvalidSubscription, ErrorInvalidWatermark
+    ErrorCannotEmptyFolder, ErrorDeleteDistinguishedFolder, ErrorInvalidSubscription, ErrorInvalidWatermark, \
+    ErrorInvalidSyncStateData
 from ..properties import FieldURI, IndexedFieldURI, ExtendedFieldURI, ExceptionFieldURI, ItemId
 from ..transport import wrap
 from ..util import chunkify, create_element, add_xml_child, get_xml_attr, to_xml, post_ratelimited, \
@@ -45,6 +46,7 @@ KNOWN_EXCEPTIONS = (
     ErrorInvalidChangeKey,
     ErrorInvalidLicense,
     ErrorInvalidSubscription,
+    ErrorInvalidSyncStateData,
     ErrorInvalidWatermark,
     ErrorItemNotFound,
     ErrorMailboxMoveInProgress,
@@ -100,6 +102,9 @@ class EWSService(metaclass=abc.ABCMeta):
         self.protocol = protocol
         # Allow a service to override the default protocol timeout. Useful for streaming services
         self.timeout = timeout
+        # Streaming connection variables
+        self._streaming_session = None
+        self._streaming_response = None
 
     # The following two methods are the minimum required to be implemented by subclasses, but the name and number of
     # kwargs differs between services. Therefore, we cannot make these methods abstract.
@@ -192,6 +197,14 @@ class EWSService(metaclass=abc.ABCMeta):
             log.debug('Processing chunk %s containing %s items', i, len(chunk))
             yield from self._get_elements(payload=payload_func(chunk, **kwargs))
 
+    def stop_streaming(self):
+        if self._streaming_response:
+            self._streaming_response.close()  # Release memory
+            self._streaming_response = None
+        if self._streaming_session:
+            self.protocol.release_session(self._streaming_session)
+            self._streaming_session = None
+
     def _get_elements(self, payload):
         """Send the payload to be sent and parsed. Handles and re-raise exceptions that are not meant to be returned
         to the caller as exception objects. Retry the request according to the retry policy.
@@ -227,11 +240,15 @@ class EWSService(metaclass=abc.ABCMeta):
                 account = self.account if isinstance(self, EWSAccountService) else None
                 log.warning('Account %s: Exception in _get_elements: %s', account, traceback.format_exc(20))
                 raise
+            finally:
+                if self.streaming:
+                    self.stop_streaming()
 
-    def _get_response_and_session(self, payload, api_version):
+    def _get_response(self, payload, api_version):
         """Send the actual HTTP request and get the response."""
         session = self.protocol.get_session()
-        return post_ratelimited(
+        self._streaming_session, self._streaming_response = None, None
+        r, session = post_ratelimited(
             protocol=self.protocol,
             session=session,
             url=self.protocol.service_endpoint,
@@ -246,6 +263,13 @@ class EWSService(metaclass=abc.ABCMeta):
             stream=self.streaming,
             timeout=self.timeout or self.protocol.TIMEOUT,
         )
+        if self.streaming:
+            # We con only release the session when we have fully consumed the response. Save session and response
+            # objects for later.
+            self._streaming_session, self._streaming_response = session, r
+        else:
+            self.protocol.release_session(session)
+        return r
 
     @property
     def _api_versions_to_try(self):
@@ -269,11 +293,7 @@ class EWSService(metaclass=abc.ABCMeta):
         log.debug('Calling service %s', self.SERVICE_NAME)
         for api_version in self._api_versions_to_try:
             log.debug('Trying API version %s', api_version)
-            r, session = self._get_response_and_session(payload=payload, api_version=api_version)
-            # TODO: We should only release the session when we have fully consumed the response, but in streaming mode
-            #  that requires fully consuming the generator returned by _get_soap_messages. The caller may not always do
-            #  that. Not doing so seems to not cause any trouble, though.
-            self.protocol.release_session(session)
+            r = self._get_response(payload=payload, api_version=api_version)
             if self.streaming:
                 # Let 'requests' decode raw data automatically
                 r.raw.decode_content = True
