@@ -1,22 +1,23 @@
-from ..util import create_element, set_xml_value, MNS
 from .common import EWSAccountService, parse_folder_elem, create_folder_ids_element
+from ..util import create_element, set_xml_value, MNS
 
 
 class CreateFolder(EWSAccountService):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/createfolder-operation"""
+
     SERVICE_NAME = 'CreateFolder'
     element_container_name = '{%s}Folders' % MNS
 
     def call(self, parent_folder, folders):
         # We can't easily find the correct folder class from the returned XML. Instead, return objects with the same
         # class as the folder instance it was requested with.
-        folders_list = list(folders)  # Convert to a list, in case 'folders' is a generator
-        for folder, elem in zip(folders_list, self._get_elements(payload=self.get_payload(
-                parent_folder=parent_folder, folders=folders
-        ))):
+        folders_list = list(folders)  # Convert to a list, in case 'folders' is a generator. We're iterating twice.
+        for folder, elem in zip(folders_list, self._chunked_get_elements(
+                self.get_payload, items=folders_list, parent_folder=parent_folder,
+        )):
             yield parse_folder_elem(elem=elem, folder=folder, account=self.account)
 
-    def get_payload(self, parent_folder, folders):
+    def get_payload(self, folders, parent_folder):
         create_folder = create_element('m:%s' % self.SERVICE_NAME)
         parentfolderid = create_element('m:ParentFolderId')
         set_xml_value(parentfolderid, parent_folder, version=self.account.version)
