@@ -1,19 +1,21 @@
+from .common import EWSService
+from ..properties import RoomList
 from ..util import create_element, MNS
 from ..version import EXCHANGE_2010
-from .common import EWSService
 
 
 class GetRoomLists(EWSService):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/getroomlists"""
+
     SERVICE_NAME = 'GetRoomLists'
     element_container_name = '{%s}RoomLists' % MNS
+    supported_from = EXCHANGE_2010
 
     def call(self):
-        from ..properties import RoomList
-
-        if self.protocol.version.build < EXCHANGE_2010:
-            raise NotImplementedError('%s is only supported for Exchange 2010 servers and later' % self.SERVICE_NAME)
         for elem in self._get_elements(payload=self.get_payload()):
+            if isinstance(elem, Exception):
+                yield elem
+                continue
             yield RoomList.from_xml(elem=elem, account=None)
 
     def get_payload(self):
