@@ -1,22 +1,23 @@
 from collections import OrderedDict
-import logging
 
+from .common import EWSAccountService, to_item_id
 from ..ewsdatetime import EWSDate
+from ..fields import FieldPath, IndexedField
+from ..indexed_properties import MultiFieldIndexedElement
+from ..properties import ItemId
 from ..util import create_element, set_xml_value, MNS
 from ..version import EXCHANGE_2013_SP1
-from .common import EWSAccountService, EWSPooledMixIn, to_item_id
-
-log = logging.getLogger(__name__)
 
 
-class UpdateItem(EWSAccountService, EWSPooledMixIn):
+class UpdateItem(EWSAccountService):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/updateitem"""
+
     SERVICE_NAME = 'UpdateItem'
     element_container_name = '{%s}Items' % MNS
 
     def call(self, items, conflict_resolution, message_disposition, send_meeting_invitations_or_cancellations,
              suppress_read_receipts):
-        from ..items import CONFLICT_RESOLUTION_CHOICES, MESSAGE_DISPOSITION_CHOICES, \
+        from ..items import Item, CONFLICT_RESOLUTION_CHOICES, MESSAGE_DISPOSITION_CHOICES, \
             SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES, SEND_ONLY
         if conflict_resolution not in CONFLICT_RESOLUTION_CHOICES:
             raise ValueError("'conflict_resolution' %s must be one of %s" % (
@@ -34,13 +35,18 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
             raise ValueError("'suppress_read_receipts' %s must be True or False" % suppress_read_receipts)
         if message_disposition == SEND_ONLY:
             raise ValueError('Cannot send-only existing objects. Use SendItem service instead')
-        return self._pool_requests(payload_func=self.get_payload, **dict(
+        for elem in self._chunked_get_elements(
+            self.get_payload,
             items=items,
             conflict_resolution=conflict_resolution,
             message_disposition=message_disposition,
             send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
             suppress_read_receipts=suppress_read_receipts,
-        ))
+        ):
+            if isinstance(elem, (Exception, type(None))):
+                yield elem
+                continue
+            yield Item.id_from_xml(elem)
 
     def _delete_item_elem(self, field_path):
         deleteitemfield = create_element('t:DeleteItemField')
@@ -88,11 +94,9 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
             value = self._get_item_value(item, field)
             if value is None or (field.is_list and not value):
                 # A value of None or [] means we want to remove this field from the item
-                for elem in self._get_delete_item_elems(field=field):
-                    yield elem
+                yield from self._get_delete_item_elems(field=field)
             else:
-                for elem in self._get_set_item_elems(item_model=item.__class__, field=field, value=value):
-                    yield elem
+                yield from self._get_set_item_elems(item_model=item.__class__, field=field, value=value)
 
     def _get_item_value(self, item, field):
         from ..items import CalendarItem
@@ -100,7 +104,7 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         if item.__class__ == CalendarItem:
             # For CalendarItem items where we update 'start' or 'end', we want to send values in the local timezone
             if field.name in ('start', 'end'):
-                if type(value) == EWSDate:
+                if type(value) is EWSDate:
                     # EWS always expects a datetime
                     return item.date_to_datetime(field_name=field.name)
                 tz_field_name = item.tz_field_for_field_name(field.name).name
@@ -108,15 +112,12 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         return value
 
     def _get_delete_item_elems(self, field):
-        from ..fields import FieldPath
         if field.is_required or field.is_required_after_save:
             raise ValueError('%s is a required field and may not be deleted' % field.name)
         for field_path in FieldPath(field=field).expand(version=self.account.version):
             yield self._delete_item_elem(field_path=field_path)
 
     def _get_set_item_elems(self, item_model, field, value):
-        from ..fields import FieldPath, IndexedField
-        from ..indexed_properties import MultiFieldIndexedElement
         if isinstance(field, IndexedField):
             # TODO: Maybe the set/delete logic should extend into subfields, not just overwrite the whole item.
             for v in value:
@@ -147,7 +148,6 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         # Takes a list of (Item, fieldnames) tuples where 'Item' is a instance of a subclass of Item and 'fieldnames'
         # are the attribute names that were updated. Returns the XML for an UpdateItem call.
         # an UpdateItem request.
-        from ..properties import ItemId
         if self.account.version.build >= EXCHANGE_2013_SP1:
             updateitem = create_element(
                 'm:%s' % self.SERVICE_NAME,
@@ -170,10 +170,11 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         itemchanges = create_element('m:ItemChanges')
         version = self.account.version
         for item, fieldnames in items:
+            if not item.account:
+                item.account = self.account
             if not fieldnames:
                 raise ValueError('"fieldnames" must not be empty')
             itemchange = create_element('t:ItemChange')
-            log.debug('Updating item %s fields %s', item, fieldnames)
             set_xml_value(itemchange, to_item_id(item, ItemId, version=version), version=version)
             updates = create_element('t:Updates')
             for elem in self._get_item_update_elems(item=item, fieldnames=fieldnames):
