@@ -1,3 +1,4 @@
+import abc
 import datetime
 from decimal import Decimal
 from keyword import kwlist
@@ -21,7 +22,7 @@ from ..common import EWSTest, get_random_string, get_random_datetime_range, get_
     get_random_decimal, get_random_choice, get_random_int, get_random_datetime
 
 
-class BaseItemTest(EWSTest):
+class BaseItemTest(EWSTest, metaclass=abc.ABCMeta):
     TEST_FOLDER = None
     FOLDER_CLASS = None
     ITEM_CLASS = None
@@ -204,13 +205,12 @@ class BaseItemTest(EWSTest):
         item_kwargs['categories'] = categories or self.categories
         return self.ITEM_CLASS(folder=folder or self.test_folder, **item_kwargs)
 
+    def get_test_folder(self, folder=None):
+        return self.FOLDER_CLASS(parent=folder or self.test_folder, name=get_random_string(8))
+
     def get_item_by_id(self, item):
-        res = list(self.account.fetch(ids=[item]))
-        assert len(res) == 1
-        res = res[0]
-        if isinstance(res, Exception):
-            raise res
-        return res
+        _id, changekey = item if isinstance(item, tuple) else (item.id, item.changekey)
+        return self.account.root.get(id=_id, changekey=changekey)
 
 
 class CommonItemTest(BaseItemTest):
@@ -251,11 +251,11 @@ class CommonItemTest(BaseItemTest):
                     # string.
                     try:
                         value_to_xml_text(filter_val)
-                    except NotImplementedError:
+                    except TypeError:
                         continue
 
                     f.is_searchable = True
-                    if f.name in ('reminder_due_by',):
+                    if f.name in ('reminder_due_by', 'conversation_index'):
                         # Filtering is accepted but doesn't work
                         self.assertEqual(
                             self.test_folder.filter(**filter_kwargs).count(),
@@ -267,81 +267,146 @@ class CommonItemTest(BaseItemTest):
                 finally:
                     f.is_searchable = False
 
-    def test_filter_on_all_fields(self):
-        # Test that we can filter on all field names
-        # TODO: Test filtering on subfields of IndexedField
-        item = self.get_test_item().save()
-        common_qs = self.test_folder.filter(categories__contains=self.categories)
-        for f in self.get_item_fields():
+    def _reduce_fields_for_filter(self, item, fields):
+        for f in fields:
             if not f.supports_version(self.account.version):
                 # Cannot be used with this EWS version
                 continue
             if not f.is_searchable:
                 # Cannot be used in a QuerySet
                 continue
-            val = getattr(item, f.name)
-            if val is None:
+            if getattr(item, f.name) is None:
                 # We cannot filter on None values
                 continue
             if self.ITEM_CLASS == Contact and f.name in ('body', 'display_name'):
                 # filtering 'body' or 'display_name' on Contact items doesn't work at all. Error in EWS?
                 continue
-            if f.is_list:
-                # Filter multi-value fields with =, __in and __contains
-                if issubclass(f.value_cls, MultiFieldIndexedElement):
-                    # For these, we need to filter on the subfield
-                    filter_kwargs = []
-                    for v in val:
-                        for subfield in f.value_cls.supported_fields(version=self.account.version):
-                            field_path = FieldPath(field=f, label=v.label, subfield=subfield)
-                            path, subval = field_path.path, field_path.get_value(item)
-                            if subval is None:
-                                continue
-                            filter_kwargs.extend([
-                                {path: subval}, {'%s__in' % path: [subval]}, {'%s__contains' % path: [subval]}
-                            ])
-                elif issubclass(f.value_cls, SingleFieldIndexedElement):
-                    # For these, we may filter by item or subfield value
-                    filter_kwargs = []
-                    for v in val:
-                        for subfield in f.value_cls.supported_fields(version=self.account.version):
-                            field_path = FieldPath(field=f, label=v.label, subfield=subfield)
-                            path, subval = field_path.path, field_path.get_value(item)
-                            if subval is None:
-                                continue
-                            filter_kwargs.extend([
-                                {f.name: v}, {path: subval},
-                                {'%s__in' % path: [subval]}, {'%s__contains' % path: [subval]}
-                            ])
+            yield f
+
+    def _run_filter_tests(self, qs, f, filter_kwargs, val):
+        for kw in filter_kwargs:
+            with self.subTest(f=f, kw=kw):
+                matches = qs.filter(**kw).count()
+                if isinstance(f, TextField) and f.is_complex:
+                    # Complex text fields sometimes fail a search using generated data. In production,
+                    # they almost always work anyway. Give it one more try after 10 seconds; it seems EWS does
+                    # some sort of indexing that needs to catch up.
+                    if not matches and isinstance(f, BodyField):
+                        # The body field is particularly nasty in this area. Give up
+                        continue
+                    for _ in range(5):
+                        if matches:
+                            break
+                        time.sleep(2)
+                        matches = qs.filter(**kw).count()
+                if f.is_list and not val and list(kw)[0].endswith('__%s' % Q.LOOKUP_IN):
+                    # __in with an empty list returns an empty result
+                    self.assertEqual(matches, 0, (f.name, val, kw))
                 else:
-                    filter_kwargs = [{'%s__in' % f.name: val}, {'%s__contains' % f.name: val}]
-            else:
-                # Filter all others with =, __in and __contains. We could have more filters here, but these should
-                # always match.
-                filter_kwargs = [{f.name: val}, {'%s__in' % f.name: [val]}]
-                if isinstance(f, TextField) and not isinstance(f, ChoiceField):
-                    # Choice fields cannot be filtered using __contains. Sort of makes sense.
-                    random_start = get_random_int(min_val=0, max_val=len(val)//2)
-                    random_end = get_random_int(min_val=len(val)//2+1, max_val=len(val))
-                    filter_kwargs.append({'%s__contains' % f.name: val[random_start:random_end]})
-            for kw in filter_kwargs:
-                with self.subTest(f=f, kw=kw):
-                    matches = common_qs.filter(**kw).count()
-                    if isinstance(f, TextField) and f.is_complex:
-                        # Complex text fields sometimes fail a search using generated data. In production,
-                        # they almost always work anyway. Give it one more try after 10 seconds; it seems EWS does
-                        # some sort of indexing that needs to catch up.
-                        if not matches:
-                            time.sleep(10)
-                            matches = common_qs.filter(**kw).count()
-                            if not matches and isinstance(f, BodyField):
-                                # The body field is particularly nasty in this area. Give up
-                                continue
-                    if f.is_list and not val and list(kw.keys())[0].endswith('__%s' % Q.LOOKUP_IN):
-                        # __in with an empty list returns an empty result
-                        self.assertEqual(matches, 0, (f.name, val, kw))
-                    else:
-                        self.assertEqual(matches, 1, (f.name, val, kw))
+                    self.assertEqual(matches, 1, (f.name, val, kw))
+
+    def test_filter_on_simple_fields(self):
+        # Test that we can filter on all simple fields
+        item = self.get_test_item()
+        fields = []
+        for f in self._reduce_fields_for_filter(item, self.get_item_fields()):
+            if f.is_list:
+                continue
+            fields.append(f)
+        if not fields:
+            self.skipTest('No matching simple fields on this model')
+        item.save()
+        common_qs = self.test_folder.filter(categories__contains=self.categories)
+        for f in fields:
+            val = getattr(item, f.name)
+            # Filter with =, __in and __contains. We could have more filters here, but these should always match.
+            filter_kwargs = [{f.name: val}, {'%s__in' % f.name: [val]}]
+            if isinstance(f, TextField) and not isinstance(f, ChoiceField):
+                # Choice fields cannot be filtered using __contains. Sort of makes sense.
+                random_start = get_random_int(min_val=0, max_val=len(val)//2)
+                random_end = get_random_int(min_val=len(val)//2+1, max_val=len(val))
+                filter_kwargs.append({'%s__contains' % f.name: val[random_start:random_end]})
+            self._run_filter_tests(common_qs, f, filter_kwargs, val)
+
+    def test_filter_on_list_fields(self):
+        # Test that we can filter on all list fields
+        item = self.get_test_item()
+        fields = []
+        for f in self._reduce_fields_for_filter(item, self.get_item_fields()):
+            if not f.is_list:
+                continue
+            if issubclass(f.value_cls, MultiFieldIndexedElement):
+                continue
+            if issubclass(f.value_cls, SingleFieldIndexedElement):
+                continue
+            fields.append(f)
+        if not fields:
+            self.skipTest('No matching list fields on this model')
+        item.save()
+        common_qs = self.test_folder.filter(categories__contains=self.categories)
+        for f in fields:
+            val = getattr(item, f.name)
+            # Filter multi-value fields with =, __in and __contains
+            filter_kwargs = [{'%s__in' % f.name: val}, {'%s__contains' % f.name: val}]
+            self._run_filter_tests(common_qs, f, filter_kwargs, val)
+
+    def test_filter_on_single_field_index_fields(self):
+        # Test that we can filter on all index fields
+        # TODO: Test filtering on subfields of IndexedField
+        item = self.get_test_item()
+        fields = []
+        for f in self._reduce_fields_for_filter(item, self.get_item_fields()):
+            if not issubclass(f.value_cls, SingleFieldIndexedElement):
+                continue
+            fields.append(f)
+        if not fields:
+            self.skipTest('No matching single index fields on this model')
+        item.save()
+        common_qs = self.test_folder.filter(categories__contains=self.categories)
+        for f in fields:
+            val = getattr(item, f.name)
+            # For these, we may filter by item or subfield value
+            filter_kwargs = []
+            for v in val:
+                for subfield in f.value_cls.supported_fields(version=self.account.version):
+                    field_path = FieldPath(field=f, label=v.label, subfield=subfield)
+                    path, subval = field_path.path, field_path.get_value(item)
+                    if subval is None:
+                        continue
+                    filter_kwargs.extend([
+                        {f.name: v}, {path: subval},
+                        {'%s__in' % path: [subval]}, {'%s__contains' % path: [subval]}
+                    ])
+            self._run_filter_tests(common_qs, f, filter_kwargs, val)
+
+    def test_filter_on_multi_field_index_fields(self):
+        # Test that we can filter on all index fields
+        # TODO: Test filtering on subfields of IndexedField
+        item = self.get_test_item()
+        fields = []
+        for f in self._reduce_fields_for_filter(item, self.get_item_fields()):
+            if not issubclass(f.value_cls, MultiFieldIndexedElement):
+                continue
+            fields.append(f)
+        if not fields:
+            self.skipTest('No matching multi index fields on this model')
+        item.save()
+        common_qs = self.test_folder.filter(categories__contains=self.categories)
+        for f in fields:
+            val = getattr(item, f.name)
+            # Filter multi-value fields with =, __in and __contains
+            # For these, we need to filter on the subfield
+            filter_kwargs = []
+            for v in val:
+                for subfield in f.value_cls.supported_fields(version=self.account.version):
+                    field_path = FieldPath(field=f, label=v.label, subfield=subfield)
+                    path, subval = field_path.path, field_path.get_value(item)
+                    if subval is None:
+                        continue
+                    filter_kwargs.extend([
+                        {path: subval}, {'%s__in' % path: [subval]}, {'%s__contains' % path: [subval]}
+                    ])
+            self._run_filter_tests(common_qs, f, filter_kwargs, val)
 
     def test_text_field_settings(self):
         # Test that the max_length and is_complex field settings are correctly set for text fields
@@ -537,7 +602,7 @@ class CommonItemTest(BaseItemTest):
         if self.ITEM_CLASS in (Contact, DistributionList):
             # Contact and DistributionList don't support mime_type updates at all
             update_kwargs.pop('mime_content', None)
-        update_fieldnames = [f for f in update_kwargs.keys() if f != 'attachments']
+        update_fieldnames = [f for f in update_kwargs if f != 'attachments']
         for k, v in update_kwargs.items():
             setattr(item, k, v)
         # Test with generator as argument
