@@ -1,8 +1,11 @@
 import datetime
+try:
+    import zoneinfo
+except ImportError:
+    from backports import zoneinfo
 
 from exchangelib.attachments import ItemAttachment
-from exchangelib.errors import ErrorItemNotFound
-from exchangelib.ewsdatetime import UTC_NOW
+from exchangelib.errors import ErrorItemNotFound, ErrorInternalServerError
 from exchangelib.extended_properties import ExtendedProperty, ExternId
 from exchangelib.fields import ExtendedPropertyField, CharField
 from exchangelib.folders import Inbox, FolderCollection
@@ -16,7 +19,8 @@ from .test_basics import CommonItemTest
 
 
 class GenericItemTest(CommonItemTest):
-    # Tests that don't need to be run for every single folder type
+    """Tests that don't need to be run for every single folder type"""
+
     TEST_FOLDER = 'inbox'
     FOLDER_CLASS = Inbox
     ITEM_CLASS = Message
@@ -34,78 +38,88 @@ class GenericItemTest(CommonItemTest):
                         setattr(item, f.name, 'a')
 
     def test_invalid_direct_args(self):
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.save()  # Must have account on save
+        item = self.get_test_item()
+        item.id = 'XXX'  # Fake a saved item
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.id = 'XXX'  # Fake a saved item
-            item.account = None
             item.save()  # Must have account on update
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.save(update_fields=['foo', 'bar'])  # update_fields is only valid on update
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.refresh()  # Must have account on refresh
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.refresh()  # Refresh an item that has not been saved
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.refresh()  # Refresh an item that doesn't exist
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.copy(to_folder=self.test_folder)  # Must have an account on copy
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.copy(to_folder=self.test_folder)  # Must be an existing item
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.copy(to_folder=self.test_folder)  # Item disappeared
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.move(to_folder=self.test_folder)  # Must have an account on move
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.move(to_folder=self.test_folder)  # Must be an existing item
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.move(to_folder=self.test_folder)  # Item disappeared
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.delete()  # Must have an account
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.delete()  # Must be an existing item
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.delete()  # Item disappeared
+        item = self.get_test_item()
+        with self.assertRaises(ValueError):
+            item._delete(delete_type='FOO', send_meeting_cancellations=None, affected_task_occurrences=None,
+                         suppress_read_receipts=None)
+        with self.assertRaises(ValueError):
+            item.delete(send_meeting_cancellations='XXX')
+        with self.assertRaises(ValueError):
+            item.delete(affected_task_occurrences='XXX')
+        with self.assertRaises(ValueError):
+            item.delete(suppress_read_receipts='XXX')
 
     def test_invalid_kwargs_on_send(self):
         # Only Message class has the send() method
@@ -163,11 +177,11 @@ class GenericItemTest(CommonItemTest):
             folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
         ).filter(categories__contains=self.categories)
         self.assertEqual(
-            [i for i in qs.order_by('subject').values_list('subject', flat=True)],
+            list(qs.order_by('subject').values_list('subject', flat=True)),
             ['Subj 0', 'Subj 1', 'Subj 2', 'Subj 3']
         )
         self.assertEqual(
-            [i for i in qs.order_by('-subject').values_list('subject', flat=True)],
+            list(qs.order_by('-subject').values_list('subject', flat=True)),
             ['Subj 3', 'Subj 2', 'Subj 1', 'Subj 0']
         )
         self.bulk_delete(qs)
@@ -185,11 +199,11 @@ class GenericItemTest(CommonItemTest):
                 folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
             ).filter(categories__contains=self.categories)
             self.assertEqual(
-                [i for i in qs.order_by('extern_id').values_list('extern_id', flat=True)],
+                list(qs.order_by('extern_id').values_list('extern_id', flat=True)),
                 ['ID 0', 'ID 1', 'ID 2', 'ID 3']
             )
             self.assertEqual(
-                [i for i in qs.order_by('-extern_id').values_list('extern_id', flat=True)],
+                list(qs.order_by('-extern_id').values_list('extern_id', flat=True)),
                 ['ID 3', 'ID 2', 'ID 1', 'ID 0']
             )
         finally:
@@ -211,28 +225,28 @@ class GenericItemTest(CommonItemTest):
                 folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
             ).filter(categories__contains=self.categories)
             self.assertEqual(
-                [i for i in qs.order_by('subject', 'extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('subject', 'extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 0', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 1'}]
             )
             self.assertEqual(
-                [i for i in qs.order_by('-subject', 'extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('-subject', 'extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 1', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 1'}]
             )
             self.assertEqual(
-                [i for i in qs.order_by('subject', '-extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('subject', '-extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 0', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 0'}]
             )
             self.assertEqual(
-                [i for i in qs.order_by('-subject', '-extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('-subject', '-extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 1', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 1'},
@@ -256,11 +270,11 @@ class GenericItemTest(CommonItemTest):
             folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
         ).filter(categories__contains=self.categories)
         self.assertEqual(
-            [i for i in qs.order_by('subject').values_list('subject', flat=True)],
+            list(qs.order_by('subject').values_list('subject', flat=True)),
             [None, None, 'Subj 0', 'Subj 2']
         )
         self.assertEqual(
-            [i for i in qs.order_by('-subject').values_list('subject', flat=True)],
+            list(qs.order_by('-subject').values_list('subject', flat=True)),
             ['Subj 2', 'Subj 0', None, None]
         )
 
@@ -280,16 +294,16 @@ class GenericItemTest(CommonItemTest):
             folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
         ).filter(subject=self.categories[0])
         self.assertEqual(
-            [i for i in qs.order_by('categories').values_list('categories', flat=True)],
+            list(qs.order_by('categories').values_list('categories', flat=True)),
             [None, None, ['Cat 0'], ['Cat 2']]
         )
         self.assertEqual(
-            [i for i in qs.order_by('-categories').values_list('categories', flat=True)],
+            list(qs.order_by('-categories').values_list('categories', flat=True)),
             [['Cat 2'], ['Cat 0'], None, None]
         )
 
     def test_finditems(self):
-        now = UTC_NOW()
+        now = datetime.datetime.now(tz=zoneinfo.ZoneInfo('UTC'))
 
         # Test argument types
         item = self.get_test_item()
@@ -620,10 +634,14 @@ class GenericItemTest(CommonItemTest):
         item.save()
         # For some reason, the querystring search doesn't work instantly. We may have to wait for up to 60 seconds.
         # I'm too impatient for that, so also allow empty results. This makes the test almost worthless but I blame EWS.
-        self.assertIn(
-            self.test_folder.filter('Subject:%s' % item.subject).count(),
-            (0, 1)
-        )
+        # Also, some servers are misconfigured and don't support querystrings at all. Don't fail on that.
+        try:
+            self.assertIn(
+                self.test_folder.filter('Subject:%s' % item.subject).count(),
+                (0, 1)
+            )
+        except ErrorInternalServerError as e:
+            self.assertIn('AQS parser has been removed from Windows 2016 Server Core', e.args[0])
 
     def test_complex_fields(self):
         # Test that complex fields can be fetched using only(). This is a test for #141.
@@ -659,7 +677,7 @@ class GenericItemTest(CommonItemTest):
 
     def test_text_body(self):
         if self.account.version.build < EXCHANGE_2013:
-            raise self.skipTest('Exchange version too old')
+            self.skipTest('Exchange version too old')
         item = self.get_test_item()
         item.body = 'X' * 500  # Make body longer than the normal 256 char text field limit
         item.save()
@@ -681,7 +699,7 @@ class GenericItemTest(CommonItemTest):
                 if f.name == 'reminder_due_by' and not item.reminder_is_set:
                     # We delete the due date if reminder is not set
                     continue
-                elif f.is_read_only:
+                if f.is_read_only:
                     continue
                 self.assertIsNotNone(getattr(item, f.name), (f, getattr(item, f.name)))
         only_fields = ('subject', 'body', 'categories')
@@ -772,6 +790,11 @@ class GenericItemTest(CommonItemTest):
         # Clean up after yourself
         del ids[3]  # Sending the deleted one through will cause an error
 
+    def test_archive(self):
+        item = self.get_test_item(folder=self.test_folder).save()
+        item_id, changekey = item.archive(to_folder=self.account.trash)
+        self.account.root.get(id=item_id, changekey=changekey)
+
     def test_item_attachments(self):
         item = self.get_test_item(folder=self.test_folder)
         item.attachments = []
